{{
*****************************************
* File jvmConvert: handling conversions *
*****************************************
}}

{{
  Conversions in the Java Virtual Machine are handled by a specific set of bytecode instructions. As you 
  know, Java allows you to convert from one type of value to another either implicitly or explicitly. When a 
  conversion occurs, such as an integer to single-precision floating-point, one of these bytecodes is used. 
  Note:
  If the conversion is to a type of smaller bit width, truncation may 
  occur. There is no notification when truncation occurs.
}}


OBJ
  js: "jvmStack"
  jd: "jvmData"
  
  
{{PUB jb_i2l}}
{{ Meaning: Integer-to-long integer conversion. }}

{{PUB jb_i2f}}
{{ Meaning: Integer to single float. }}

{{PUB jb_i2d}}
{{ Meaning: Integer to double float. }}

{{PUB jb_l2i}}
{{ Meaning: Long integer to integer. }}

{{PUB jb_l2f}}
{{ Meaning: Long integer to single float. }}

{{PUB jb_l2d}}
{{ Meaning: Long integer to double float. }}

{{PUB jb_f2i}}
{{ Meaning: Single float to integer. }}

{{PUB jb_f2l}}
{{ Meaning: Single float to long integer. }}

{{PUB jb_f2d}}
{{ Meaning: Single float to double float. }}

{{PUB jb_d2i}}
{{ Meaning: Double float to integer. }}

{{PUB jb_d2l}}
{{ Meaning: Double float to long integer. }}

{{PUB jb_d2f}}
{{ Meaning: Double float to single float. }}


PUB i2b | w1
{{ Auxiliary function (no bytecode): Meaning: Integer to signed byte. }}
  w1 := js.pop
  w1 &= $FF
  js.push(w1)
  

PUB jb_i2b | w1
{{ Meaning: Integer to signed byte. }}
{{
  JEM format: i2b
}}
  i2b
  jd.incJvmPC(1)
  

PUB i2c | w1
{{ Auxiliary function (no bytecode): Meaning: Integer to char. }}
  w1 := js.pop
  w1 &= $FF
  js.push(w1)


PUB jb_i2c | w1
{{ Meaning: Integer to char. }}
{{
  JEM format: i2c
}}
  i2c
  jd.incJvmPC(1)


{{PUB jb_i2s}}
{{ Meaning: Integer to short. }}


PUB b2i(bytevalue): value
{{ Auxiliary function (no bytecode): Meaning: Signed byte to integer. }}
  return ~bytevalue 'b7 sign extend
  

PUB c2i(charvalue): value
{{ Auxiliary function (no bytecode): Meaning: Char to integer. }}
  return charvalue & $FF
  

PUB z2i(booleanvalue): value
{{ Auxiliary function (no bytecode): Meaning: Boolean to integer. }}
  if booleanvalue == 0
    return 0
  return -1
  

{{PUB jb_s2i}}
{{ Auxiliary function (no bytecode): Meaning: Short to integer. }}