{{
**********************************
* jvmEngine - executes bytecodes *
**********************************
}}


CON
  'first 64
  Jem_NOP               = $00
  Jem_ICONST_M1         = $01
  Jem_ICONST_0          = $02
  Jem_ICONST_1          = $03
  Jem_ICONST_2          = $04
  Jem_ICONST_3          = $05
  Jem_ICONST_4          = $06
  Jem_ICONST_5          = $07
  Jem_BIPUSH            = $08
  Jem_SIPUSH            = $09
  Jem_ILOAD             = $0A
  Jem_ILOAD_0           = $0B
  Jem_ILOAD_1           = $0C
  Jem_ILOAD_2           = $0D
  Jem_ILOAD_3           = $0E
  Jem_ISTORE            = $0F
  Jem_ISTORE_0          = $10
  Jem_ISTORE_1          = $11
  Jem_ISTORE_2          = $12
  Jem_ISTORE_3          = $13
  Jem_IFEQ              = $14
  Jem_IFNE              = $15
  Jem_IFLT              = $16
  Jem_IFLE              = $17
  Jem_IFGT              = $18
  Jem_IFGE              = $19
  Jem_IF_ICMPEQ         = $1A
  Jem_IF_ICMPNE         = $1B
  Jem_IF_ICMPLT         = $1C
  Jem_IF_ICMPGE         = $1D
  Jem_IF_ICMPGT         = $1E
  Jem_IF_ICMPLE         = $1F
  Jem_IF_ACMPEQ         = $20
  Jem_IF_ACMPNE         = $21
  Jem_IRETURN           = $22
  Jem_RETURN            = $23
  Jem_INVOKEVIRTUAL     = $24
  Jem_INVOKESTATIC      = $25
  Jem_GETSTATIC         = $26
  Jem_PUTSTATIC         = $27
  Jem_POP               = $28
  Jem_POP2              = $29
  Jem_DUP               = $2A
  Jem_DUP_X1            = $2B
  Jem_DUP_X2            = $2C
  Jem_DUP2              = $2D
  Jem_DUP2_X1           = $2E
  Jem_DUP2_X2           = $2F
  Jem_SWAP              = $30
  Jem_IADD              = $31
  Jem_ISUB              = $32
  Jem_INEG              = $33
  Jem_IOR               = $35
  Jem_ISHL              = $37
  Jem_ISHR              = $39
  Jem_IUSHR             = $3B
  Jem_IAND              = $3D
  Jem_IXOR              = $3F
  Jem_I2B               = $41
  Jem_GOTO              = $43
  Jem_INVOKEINTERFACE   = $45
  Jem_ARRAYLENGTH       = $47
  Jem_ATHROW            = $49
  Jem_CHECKCAST         = $4B
  'next 64
  Jem_INSTANCEOF        = $4D
  Jem_WIDE              = $4F
  Jem_IINC              = $51
  Jem_GETFIELD          = $53
  Jem_PUTFIELD          = $55
  Jem_NEW               = $57
  Jem_NEWARRAY          = $59
  Jem_ANEWARRAY         = $5B
  Jem_IALOAD            = $5D
  Jem_IASTORE           = $5F
  Jem_BALOAD            = $61
  Jem_BASTORE           = $63
  Jem_LDC               = $65
  Jem_LDC_W             = $67
  Jem_LDC_W2            = $69
  Jem_IMUL              = $6B
  Jem_IDIV              = $6D
  Jem_IREM              = $6F
  Jem_JSR               = $71
  Jem_RET               = $73
  Jem_TABLESWITCH       = $75
  Jem_LOOKUPSWITCH      = $77


OBJ
  comm:          "jvmComm"    'communication port object
  jd:            "jvmData"    'jvm memory access object

  jStack:        "jvmStack"
  jConst:        "jvmConst"
  jLoad:         "jvmLoad"
  jStore:        "jvmStore"
  jCompareJump:  "jvmCompareJump"
  jInvoke:       "jvmInvoke"
  jMath:         "jvmMath"
  jArray:        "jvmArray"
  jObject:       "jvmObject"
  jSubReturn:    "jvmSubReturn"
  jLogic:        "jvmLogic"
  jConvert:      "jvmConvert"
  
   
PUB doBytecode: value | state,c
{{ Execute a bytecode }}
  state := jd.getActivity
  case state
    jd#kRun    : 'JVM running
                 c := doStep
                 if jd.mustNotifyStepDone
                   comm.notifyStepDone
                 return -32768
    jd#kBreak  : 'JVM reached breakpoint
                 'wait for IDE to send debug command (run/step/stop)
                 return -32768
    jd#kPassive: 'JVM was running but has stopped (exception,stopped,end)
                 if jd.mustStep
                   c := doStep
                   if jd.mustNotifyStepDone
                     comm.notifyStepDone
                 return -32768
    jd#kJog    : 'appears not to be used
                 'so change state to kPassive
                 jd.setActivity(jd#kPassive)
                 return -32768 
    
  return -32768

          
PRI doStep: value | jemcode
{{ perform a step }}
  jemcode := jd.readJavaByte(jd.getJvmPC)
  if jemcode < Jem_INSTANCEOF
    case jemcode
      Jem_NOP:             jStack.jb_nop
      Jem_ICONST_M1:       jConst.jb_iconst_m1
      Jem_ICONST_0:        jConst.jb_iconst_0
      Jem_ICONST_1:        jConst.jb_iconst_1
      Jem_ICONST_2:        jConst.jb_iconst_2
      Jem_ICONST_3:        jConst.jb_iconst_3
      Jem_ICONST_4:        jConst.jb_iconst_4
      Jem_ICONST_5:        jConst.jb_iconst_5
      Jem_BIPUSH:          jConst.jb_bipush
      Jem_SIPUSH:          jConst.jb_sipush
      Jem_ILOAD:           jLoad.jb_iload
      Jem_ILOAD_0:         jLoad.jb_iload_0
      Jem_ILOAD_1:         jLoad.jb_iload_1
      Jem_ILOAD_2:         jLoad.jb_iload_2
      Jem_ILOAD_3:         jLoad.jb_iload_3
      Jem_ISTORE:          jStore.jb_istore
      Jem_ISTORE_0:        jStore.jb_istore_0
      Jem_ISTORE_1:        jStore.jb_istore_1
      Jem_ISTORE_2:        jStore.jb_istore_2
      Jem_ISTORE_3:        jStore.jb_istore_3
      Jem_IFEQ:            jCompareJump.jb_ifeq
      Jem_IFNE:            jCompareJump.jb_ifne
      Jem_IFLT:            jCompareJump.jb_iflt
      Jem_IFLE:            jCompareJump.jb_ifle
      Jem_IFGT:            jCompareJump.jb_ifgt
      Jem_IFGE:            jCompareJump.jb_ifge
      Jem_IF_ICMPEQ:       jCompareJump.jb_if_icmpeq
      Jem_IF_ICMPNE:       jCompareJump.jb_if_icmpne
      Jem_IF_ICMPLT:       jCompareJump.jb_if_icmplt
      Jem_IF_ICMPGE:       jCompareJump.jb_if_icmpge
      Jem_IF_ICMPGT:       jCompareJump.jb_if_icmpgt
      Jem_IF_ICMPLE:       jCompareJump.jb_if_icmple
      Jem_IF_ACMPEQ:       jCompareJump.jb_if_acmpeq
      Jem_IF_ACMPNE:       jCompareJump.jb_if_acmpne
      Jem_IRETURN:         jSubReturn.jb_ireturn
      Jem_RETURN:          jSubReturn.jb_return
      Jem_INVOKEVIRTUAL:   jInvoke.jb_invokevirtual
      Jem_INVOKESTATIC:    jInvoke.jb_invokestatic
      Jem_GETSTATIC:       jObject.jb_getstatic
      Jem_PUTSTATIC:       jObject.jb_putstatic
      Jem_POP:             jStack.jb_pop
      Jem_POP2:            jStack.jb_pop2
      Jem_DUP:             jStack.jb_dup
      Jem_DUP_X1:          jStack.jb_dup_x1
      Jem_DUP_X2:          jStack.jb_dup_x2
      Jem_DUP2:            jStack.jb_dup2
      Jem_DUP2_X1:         jStack.jb_dup2_x1
      Jem_DUP2_X2:         jStack.jb_dup2_x2
      Jem_SWAP:            jStack.jb_swap
      Jem_IADD:            jMath.jb_iadd
      Jem_ISUB:            jMath.jb_isub
      Jem_INEG:            jMath.jb_ineg
      Jem_IOR:             jLogic.jb_ior
      Jem_ISHL:            jLogic.jb_ishl
      Jem_ISHR:            jLogic.jb_ishr
      Jem_IUSHR:           jLogic.jb_iushr
      Jem_IAND:            jLogic.jb_iand
      Jem_IXOR:            jLogic.jb_ixor
      Jem_I2B:             jConvert.jb_i2b
      Jem_GOTO:            jCompareJump.jb_goto
      Jem_INVOKEINTERFACE: 'not used by linker
      Jem_ARRAYLENGTH:     jArray.jb_arraylength
      Jem_ATHROW:          jInvoke.jb_athrow
      Jem_CHECKCAST:       jObject.jb_checkcast
      other:

  else
    case jemcode
      Jem_INSTANCEOF:      jObject.jb_instanceof
      Jem_WIDE:            'not used by linker
      Jem_IINC:            jStore.jb_iinc
      Jem_GETFIELD:        jObject.jb_getfield
      Jem_PUTFIELD:        jObject.jb_putfield
      Jem_NEW:             jObject.jb_new
      Jem_NEWARRAY:        jArray.jb_newarray
      Jem_ANEWARRAY:       jArray.jb_anewarray
      Jem_IALOAD:          jArray.jb_iaload
      Jem_IASTORE:         jArray.jb_iastore
      Jem_BALOAD:          jArray.jb_baload
      Jem_BASTORE:         jArray.jb_bastore
      Jem_LDC:             jConst.jb_ldc
      Jem_LDC_W:           jConst.jb_ldc_w
      Jem_LDC_W2:          'not used by linker
      Jem_IMUL:            jMath.jb_imul
      Jem_IDIV:            jMath.jb_idiv
      Jem_IREM:            jMath.jb_irem
      Jem_JSR:             'not used by linker
      Jem_RET:             'not used by linker
      Jem_TABLESWITCH:     jCompareJump.jb_tableswitch
      Jem_LOOKUPSWITCH:    jCompareJump.jb_lookupswitch
      other:

      